dofile("sys/lua/zombies/utils.lua")
dofile("sys/lua/zombies/customnpcs.lua")
dofile("sys/lua/zombies/items.lua")
dofile("sys/lua/zombies/chests.lua")
dofile("sys/lua/zombies/time.lua")
dofile("sys/lua/zombies/crafting.lua")
dofile("sys/lua/zombies/towns.lua")
dofile("sys/lua/zombies/monsters.lua")
dofile("sys/lua/zombies/mining.lua")

parse("mp_hud 77") -- Display only health and ammo

zombies = {}

zombies.internal = {}
zombies.internal.onmap = 0
zombies.internal.zombies = {}
zombies.players = {}
zombies.buffs = {}
zombies.actions = {}
zombies.flashtimer = 0
zombies.beds = {}
zombies.godmode = {}
zombies.away = {}
zombies.hudtxtoffset = 10

-- FUNCTIONS --

function zombies.addHunger(id,hunger) -- Adds hunger to the hunger value of a player. hunger can be negative to subtract hunger
	local usgn=player(id,"usgn")
	if zombies.players[usgn].hunger+hunger<=zombies.maxhunger then
		if zombies.players[usgn].hunger+hunger>=0 then
			zombies.players[usgn].hunger=zombies.players[usgn].hunger+hunger
		else
			zombies.players[usgn].hunger=0
		end
	else
		zombies.players[usgn].hunger=zombies.maxhunger
	end
end

function zombies.addThirst(id,thirst) -- Adds thirst to the thirst value of a player. thirst can be negative to subtract thirst
	local usgn=player(id,"usgn")
	if zombies.players[usgn].thirst+thirst<=zombies.maxthirst then
		if zombies.players[usgn].thirst+thirst>=0 then
			zombies.players[usgn].thirst=zombies.players[usgn].thirst+thirst
		else
			zombies.players[usgn].thirst=0
		end
	else
		zombies.players[usgn].thirst=zombies.maxthirst
	end
end

function zombies.addFatigue(id,fatigue) -- Adds fatigue to the fatigue value of a player. fatigue can be negative to subtract fatigue
	local usgn=player(id,"usgn")
	if zombies.players[usgn].fatigue+fatigue<=zombies.maxfatigue then
		if zombies.players[usgn].fatigue+fatigue>=0 then
			zombies.players[usgn].fatigue=zombies.players[usgn].fatigue+fatigue
		else
			zombies.players[usgn].fatigue=0
		end
	else
		zombies.players[usgn].fatigue=zombies.maxfatigue
	end
end

function zombies.addRadiation(id,radiation) -- Adds radiation to the rads value of a player. radiation can be negative to subtract rads
	local usgn=player(id,"usgn")
	zombies.players[usgn].rads = zombies.players[usgn].rads + radiation
end

function zombies.addHealth(id,health) -- Adds health to the health value of a player. health can be negative to subtract health
	local usgn=player(id,"usgn")
	if zombies.players[usgn].health+health<=zombies.maxhealth then
		if zombies.players[usgn].health+health>=0 then
			zombies.players[usgn].health=zombies.players[usgn].health+health
		else
			zombies.players[usgn].health=0
		end
	else
		zombies.players[usgn].health=zombies.maxhealth
	end
	parse("sethealth "..id.." "..math.ceil(zombies.players[usgn].health))
end

--[[
Deals dmg damage to a player. Some of the damage will be absorbed by the player's defence.

Use zombies.addHealth with a negative value to deal pure damage.
]]
function zombies.damage(id,dmg)
	local usgn=player(id,"usgn")
	local def=0
	def=def+zombies.defperstat[1]*zombies.players[player(id,"usgn")].hunger -- Add the defence given by the hunger stat
	def=def+zombies.defperstat[2]*zombies.players[player(id,"usgn")].thirst -- Add the defence given by the thirst stat
	def=def+zombies.defperstat[3]*zombies.players[player(id,"usgn")].fatigue -- Add the defence given by the fatigue stat
	dmg=dmg-(dmg*def) -- Absorb damage
	if not zombies.godmode[id] and not zombies.away[id] then
		if dmg<zombies.getHealth(id) then
			zombies.players[usgn].health=zombies.players[usgn].health-dmg -- Subtract health
		else
			zombies.players[usgn].health=0
		end
		parse("sethealth "..id.." "..zombies.getHealth(id)) -- Set health in CS2D
	end
end

--[[
Creates a buff (effect) archetype with the ID buff and the name name.

funcstable contains functions that will be called at certain events - it can have fields that correspond to some of the hooks:
ms100(id)
second(id)
minute(id)
dmgrecv(src,id)
dmgdeal(id,src)
movetile(id,x,y)
add(id,buff,duration)
remove(id,buff)
zombiekill(id,zid)
zombiedmg(id,zid,dmg)

]]
function zombies.createBuff(buff,name,funcstable)
	zombies.buffs[buff]={}
	zombies.buffs[buff].name = name
	for kf,f in pairs(funcstable) do
		zombies.buffs[buff][kf]=f
	end
end

function zombies.addBuffFunc(buff,funckey,func) -- Adds a function to the buff archetype
	zombies.players[usgn].buffs[buff][funckey]=func
end

function zombies.addBuff(id,buff,duration) -- Adds a buff to a player
	local usgn=player(id,"usgn")
	for _, b in pairs(zombies.players[usgn].buffs) do
		if b[1]==buff then
			b[2] = b[2] + duration
			if zombies.buffs[buff].add then zombies.buffs[buff].add(id,buff,duration) end
			return true
		end
	end
	zombies.players[usgn].buffs[#zombies.players[usgn].buffs+1]={buff,duration}
	if zombies.buffs[buff].add then zombies.buffs[buff].add(id,buff,duration) end
	return true
end

function zombies.removeBuff(id,buff) -- Removes a buff from a player
	local usgn=player(id,"usgn")
	if zombies.buffs[buff].remove then zombies.buffs[buff].remove(id,buff) end
	for bk,b in pairs(zombies.players[usgn].buffs) do
		if b.buffid==buff then
			table.remove(zombies.players[usgn].buffs,bk)
		end
	end
end

function zombies.getHunger(id) -- Returns the hunger value of a player
	return zombies.players[player(id,"usgn")].hunger
end

function zombies.getThirst(id) -- Returns the thirst value of a player
	return zombies.players[player(id,"usgn")].thirst
end

function zombies.getFatigue(id) -- Returns the fatigue value of a player
	return zombies.players[player(id,"usgn")].fatigue
end

function zombies.getHealth(id) -- Returns the health value of a player
	return zombies.players[player(id,"usgn")].health
end

function zombies.addAction(x,y,func) -- func will be called when a player presses Use on tile X|Y, the player ID will be passed to it as an argument 
	zombies.actions[#zombies.actions+1]={x,y,func}
end

function zombies.createBed(x,y,rx,ry,img,rot)
	img = img or "gfx/customnpcs/bed.png"
	rot = rot or 0
	local imgid = image(img,0,0,0)
	imagepos(imgid,x,y,rot)
	zombies.beds[#zombies.beds+1]={x,y,rx,ry,imgid}
end

function zombies.foundTown(id,plot,name)
	if (zombies.players[player(id,"usgn")].rank >= towns.townrank) then
		if not towns.internal.plotocc[plot] then
			towns.createTown(plot,player(id,"usgn"),name,{})
			msg(player(id,"name").." has founded "..name)
		else
			msg2(id,"This plot is already occupied!")
		end
	else
		msg2(id,"You need to be of a higher rank!")
	end
end

-- INTERNAL FUNCTIONS --

addhook("spawn","zombies.internal.spawn")
function zombies.internal.spawn(id)
	parse("sethealth "..id.." "..zombies.players[player(id,"usgn")].health)
	if zombies.players[player(id,"usgn")].spawnpos then
		parse("setpos "..id.." "..zombies.players[player(id,"usgn")].spawnpos[1].." "..zombies.players[player(id,"usgn")].spawnpos[2])
	end
	parse("setarmor "..id.." "..(zombies.players[player(id,"usgn")].armour or 0))
	if not zombies.players[player(id,"usgn")].weapons then zombies.players[player(id,"usgn")].weapons = {} end
	if #zombies.players[player(id,"usgn")].weapons == 0 then
		return "x"
	else
		return table.concat(zombies.players[player(id,"usgn")].weapons,",")
	end
end

addhook("die","zombies.internal.die")
function zombies.internal.die(id,src)
	zombies.players[player(id,"usgn")].health=100
	if zombies.away[id] then
		zombies.away[id] = false
		msg("\169000255000"..player(id,"name").."\169255220000 is no longer away.")
	end
	if not zombies.godmode[id] and not zombies.away[id] then
		if zombies.loseInventory then
			items.clearInventory(id)
			zombies.players[player(id,"usgn")].weapons = {}
			zombies.players[player(id,"usgn")].armour = 0
		end
	end
end

addhook("serveraction","zombies.internal.serveraction")
function zombies.internal.serveraction(id,a)
	if not zombies.away[id] and not zombies.players[player(id,"usgn")].sleeping then
		if a==1 then
			items.openInventory(id)
		end
		if a==2 then
			crafting.displayRecipes(id)
		end
		if a==3 then
			if items.internal.quickuse[id] then
				items[items.internal.quickuse[id]][3](id,items.internal.quickuse[id])
			end
		end
	end
end

addhook("ms100","zombies.internal.ms100")
function zombies.internal.ms100()
	for _,id in pairs(player(0,"tableliving")) do
		for _,b in pairs(zombies.players[player(id,"usgn")].buffs) do
			if zombies.buffs[b[1]].ms100 then zombies.buffs[b[1]].ms100(id) end
		end
	end
end

function zombies.spawnEnemy(c)
	if zombies.internal.onmap<zombies.mapamount then
		local x, y, rot = math.random(0,map("xsize")), math.random(0,map("ysize")), math.random(0,360)
		for _,sz in pairs(zombies.safezones) do
			if not tile(x,y,"walkable") or (zombies.nospawnframes and table.contains(zombies.nospawnframes,tile(x,y,"frame"))) or x>=sz[1] and y>=sz[2] and x<=sz[3] and y<=sz[4] then
				zombies.spawnEnemy(c)
				return
			end
		end
		--parse("spawnnpc "..c[2].." "..x.." "..y.." "..rot)
		monsters.spawnMonster(c[2],x*32+16,y*32+16,rot)
		--zombies.internal.zombies[object(0,"table")[#object(0,"table")]] = {100,c[2]}
		zombies.internal.onmap = zombies.internal.onmap+1
		return true
	end
end

zombies.flashdmg = 20
local flashimg = image("gfx/sprites/block.bmp",320,240,2)
imagealpha(flashimg,0)
imagecolor(flashimg,255,255,255)
imagescale(flashimg,20,15)

function zombies.flash()
	msg("There is a flash in the sky! Seek cover immediately!")
	timer(5000,"parse","lua tween_alpha("..flashimg..",1000,0.7)")
	timer(5000,"zombies.internal.flashdmgf")
	timer(6000,"parse","lua tween_color("..flashimg..",1000,0,0,0)")
	timer(6500,"parse","lua tween_alpha("..flashimg..",1000,0)")
	timer(7500,"parse","lua imagecolor("..flashimg..",255,255,255)")
end

function zombies.internal.flashdmgf()
	for _,id in pairs(player(0,"tableliving")) do
		local incover = false
		for _,cz in pairs(zombies.cover) do
			if player(id,"tilex")>=cz[1] and player(id,"tiley")>=cz[2] and player(id,"tilex")<=cz[3] and player(id,"tiley")<=cz[4] then
				incover = true
				break
			end
		end
		if not incover and not zombies.godmode[id] then
			if zombies.getHealth(id)>zombies.flashdmg then
				zombies.damage(id,zombies.flashdmg)
			else
				parse("customkill 0 \"Solar Flare\""..id)
			end
		end
	end
end

addhook("clientdata","zombies.internal.clientdata")
function zombies.internal.clientdata(id,mode,a1,a2)
	if mode==2 then
		if not zombies.players[player(id,"usgn")].cursor then zombies.players[player(id,"usgn")].cursor={} end
		zombies.players[player(id,"usgn")].cursor[1],zombies.players[player(id,"usgn")].cursor[2]=a1,a2
	end
end

addhook("attack","zombies.internal.attack")
function zombies.internal.attack(id)
	if not zombies.godmode[id] and not zombies.away[id] then
		zombies.addHunger(id,zombies.statsperatk[1])
		zombies.addThirst(id,zombies.statsperatk[2])
		zombies.addFatigue(id,zombies.statsperatk[3])
	end
	if zombies.away[id] then
		zombies.away[id] = false
		msg("\169000255000"..player(id,"name").."\169255220000 is no longer away.")
	end
end

addhook("move","zombies.internal.pixelmove")
function zombies.internal.pixelmove(id)
	if zombies.away[id] then
		zombies.away[id] = false
		msg("\169000255000"..player(id,"name").."\169255220000 is no longer away.")
	end
end

addhook("attack2","zombies.internal.attack2")
function zombies.internal.attack2(id)
	if zombies.players[player(id,"usgn")].building then
		reqcld(id,2)
		local taken=items.takeItems(id,zombies.buildingprice[zombies.players[player(id,"usgn")].building])
		if taken then
			towns.internal.buildattempt(id,zombies.players[player(id,"usgn")].building,math.floor(zombies.players[player(id,"usgn")].cursor[1]/32),math.floor(zombies.players[player(id,"usgn")].cursor[2]/32),0)
		else
			msg2(id,string.char(169).."255000000You don't have the required materials!")
			zombies.players[player(id,"usgn")].building=nil
		end
	end
	if zombies.away[id] then
		zombies.away[id] = false
		msg("\169000255000"..player(id,"name").."\169255220000 is no longer away.")
	end
end

addhook("second","zombies.internal.second")
function zombies.internal.second()
	for _,c in pairs(zombies.spawnchances) do
		local randchance = math.random(1,100)
		if randchance>100-c[1] then
			zombies.spawnEnemy(c)
		end
	end
	parse("hudtxt ".. zombies.hudtxtoffset+6 .." \"Enemies: "..zombies.internal.onmap.."\" 320 16 1")
	
	zombies.flashtimer = zombies.flashtimer + 1
	if zombies.flashtimer>=zombies.flashtime then
		local randchance = math.random(1,100)
		if randchance>100-zombies.flashchance then
			zombies.flash()
			zombies.flashtimer = 0
		end
	end
	
	for _,id in pairs(player(0,"tableliving")) do
		
		parse("hudtxt2 "..id.." "..zombies.buffhudoffset.." \"Effects:\" 100 100")
		
		local usgn = player(id,"usgn")
		local rads = 0
		if (type(zombies.radspersec)=="table") then
			for _, rad in pairs(zombies.radspersec) do
				if time[1]>=rad[1] and time[1]<=rad[2] then
					rads = rad[3]
				end
			end
		elseif (type(zombies.radspersec)=="number") then
			rads = zombies.radspersec
		end
		if zombies.radsmults then
			for _, reg in pairs(zombies.radsmults) do
				if coordsInBounds(player(id,"tilex"),player(id,"tiley"),reg) then
					rads = rads * reg[5]
				end
			end
		end
		
		rads = rads * zombies.radmultarmor[zombies.players[usgn].armour]
		
		zombies.players[usgn].radspersec = rads
		
		for _,b in pairs(zombies.players[player(id,"usgn")].buffs) do
			if zombies.buffs[b[1]].second then zombies.buffs[b[1]].second(id) end
			if b[2] then
				if b[2]>1 then
					b[2]=b[2]-1
					local m=tostring(math.modf(b[2]/60))
					local s=tostring(b[2]%60)
					if #s==1 then
						s="0"..s
					end
					parse("hudtxt2 "..id.." "..zombies.buffhudoffset+_.." \""..zombies.buffs[b[1]].name.." ("..m..":"..s..")\" 100 "..(100+(16*_)))
				else
					table.remove(zombies.players[player(id,"usgn")].buffs,_)
					parse("hudtxt2 "..id.." "..zombies.buffhudoffset+_.." \"\" 100 "..(100+(16*_)))
				end
			end
		end
		
		if not zombies.godmode[id] and not zombies.away[id] then
			
			zombies.addRadiation(id,zombies.players[usgn].radspersec)
			
			if zombies.getHunger(id)>zombies.regthreshold[1] then
				zombies.addHealth(id,zombies.regperstatsec[1])
			end
			
			if zombies.getThirst(id)>zombies.regthreshold[2] then
				zombies.addHealth(id,zombies.regperstatsec[2])
			end
			
			if zombies.getFatigue(id)>zombies.regthreshold[3] then
				zombies.addHealth(id,zombies.regperstatsec[3])
			end
			
			if zombies.getHunger(id)<zombies.dmgthreshold[1] then
				zombies.addHealth(id,-zombies.dmgperstatsec[1])
			end
			
			if zombies.getThirst(id)<zombies.dmgthreshold[2] then
				zombies.addHealth(id,-zombies.dmgperstatsec[2])
			end
			
			if zombies.getFatigue(id)<zombies.dmgthreshold[3] then
				zombies.addHealth(id,-zombies.dmgperstatsec[3])
			end
		
			zombies.addHunger(id,zombies.statspersec[1])
			zombies.addThirst(id,zombies.statspersec[2])
			zombies.addFatigue(id,zombies.statspersec[3])
		
			local x, y = player(id,"tilex"), player(id,"tiley")
			for _,sz in pairs(zombies.safezones) do
				if x>=sz[1] and y>=sz[2] and x<=sz[3] and y<=sz[4] then
					zombies.addHunger(id,zombies.statsperssz[1])
					zombies.addThirst(id,zombies.statsperssz[2])
					zombies.addFatigue(id,zombies.statsperssz[3])
				end
			end
			
			if zombies.players[player(id,"usgn")].sleeping then
				zombies.addHunger(id,zombies.statspersib[1])
				zombies.addThirst(id,zombies.statspersib[2])
				zombies.addFatigue(id,zombies.statspersib[3])
			end
			
			zombies.addHunger(id,zombies.statsperrad[1]*zombies.players[player(id,"usgn")].rads)
			zombies.addThirst(id,zombies.statsperrad[2]*zombies.players[player(id,"usgn")].rads)
			zombies.addFatigue(id,zombies.statsperrad[3]*zombies.players[player(id,"usgn")].rads)
			zombies.addHealth(id,zombies.statsperrad[4]*zombies.players[player(id,"usgn")].rads)
		end
		
		local hcolours={0,zombies.maxindcolour}
		hcolours[1]=tostring(math.floor(zombies.maxindcolour-(zombies.maxindcolour/zombies.maxhunger*zombies.players[player(id,"usgn")].hunger)))
		hcolours[2]=tostring(math.floor(zombies.maxindcolour/zombies.maxhunger*zombies.players[player(id,"usgn")].hunger))
		local tcolours={0,zombies.maxindcolour}
		tcolours[1]=tostring(math.floor(zombies.maxindcolour-(zombies.maxindcolour/zombies.maxthirst*zombies.players[player(id,"usgn")].thirst)))
		tcolours[2]=tostring(math.floor(zombies.maxindcolour/zombies.maxthirst*zombies.players[player(id,"usgn")].thirst))
		local fcolours={0,zombies.maxindcolour}
		fcolours[1]=tostring(math.floor(zombies.maxindcolour-(zombies.maxindcolour/zombies.maxfatigue*zombies.players[player(id,"usgn")].fatigue)))
		fcolours[2]=tostring(math.floor(zombies.maxindcolour/zombies.maxfatigue*zombies.players[player(id,"usgn")].fatigue))
		
		if #hcolours[1]<3 then hcolours[1]=("0"):rep(3-#hcolours[1])..hcolours[1] end
		if #hcolours[2]<3 then hcolours[2]=("0"):rep(3-#hcolours[2])..hcolours[2] end
		
		if #tcolours[1]<3 then tcolours[1]=("0"):rep(3-#tcolours[1])..tcolours[1] end
		if #tcolours[2]<3 then tcolours[2]=("0"):rep(3-#tcolours[2])..tcolours[2] end
		
		if #fcolours[1]<3 then fcolours[1]=("0"):rep(3-#fcolours[1])..fcolours[1] end
		if #fcolours[2]<3 then fcolours[2]=("0"):rep(3-#fcolours[2])..fcolours[2] end
		
		parse("hudtxt2 "..id.." ".. zombies.hudtxtoffset+3 .." \""..string.char(169)..hcolours[1]..hcolours[2].."000Hunger: "..math.ceil(zombies.players[player(id,"usgn")].hunger).."\" 320 420 1")
		parse("hudtxt2 "..id.." ".. zombies.hudtxtoffset+4 .." \""..string.char(169)..tcolours[1]..tcolours[2].."000Thirst: "..math.ceil(zombies.players[player(id,"usgn")].thirst).."\" 320 436 1")
		parse("hudtxt2 "..id.." ".. zombies.hudtxtoffset+5 .." \""..string.char(169)..fcolours[1]..fcolours[2].."000Fatigue: "..math.ceil(zombies.players[player(id,"usgn")].fatigue).."\" 320 452 1")
	end
end

function zombies.killZombie(m,id)
	if zombies.drops[m.archetype] then
		for _,d in pairs(zombies.drops[m.archetype]) do
			local randchance = math.random(1,100)
			if randchance <= d[1] then
				items.addToInventory(id,d[2],d[3])
				if d[3] == 1 then
					msg2(id,"You found a "..items[d[2]][1].."!")
				else
					msg2(id,"You found "..d[3].." "..items[d[2]][1].."s!")
				end
			end
		end
	end
	for _,b in pairs(zombies.players[player(id,"usgn")].buffs) do
		if zombies.buffs[b[1]].zombiekill then zombies.buffs[b[1]].zombiekill(id,m.archetype) end
	end
	zombies.players[player(id,"usgn")].kills=zombies.players[player(id,"usgn")].kills+1
	for _,r in pairs(zombies.ranks) do
		if zombies.players[player(id,"usgn")].kills==r[1] then
			zombies.players[player(id,"usgn")].rank=_
			msg(string.char(169).."000255000"..player(id,"name").." "..string.char(169).."255220000has been promoted to "..r[2])
		end
	end
	zombies.internal.onmap=zombies.internal.onmap-1
	parse("hudtxt ".. zombies.hudtxtoffset+6 .." \""..string.char(169).."155000000Enemies: "..zombies.internal.onmap.."\" 320 16 1")
end

addhook("objectkill","zombies.internal.objectkill")
function zombies.internal.objectkill(oid,id)
	if id~=0 then
		if object(oid,"type")==30 then
			for _,id in pairs(player(0,"tableliving")) do
				for _,b in pairs(zombies.players[player(id,"usgn")].buffs) do
					if zombies.buffs[b[1]].zombiekill then zombies.buffs[b[1]].zombiekill(id,oid) end
				end
			end
			zombies.players[player(id,"usgn")].kills=zombies.players[player(id,"usgn")].kills+1
			for _,r in pairs(zombies.ranks) do
				if zombies.players[player(id,"usgn")].kills==r[1] then
					zombies.players[player(id,"usgn")].rank=_
					msg(string.char(169).."000255000"..player(id,"name").." "..string.char(169).."255220000has been promoted to "..r[2])
				end
			end
			zombies.internal.onmap=zombies.internal.onmap-1
			parse("hudtxt ".. zombies.hudtxtoffset+6 .." \""..string.char(169).."155000000Enemies: "..zombies.internal.onmap.."\" 320 16 1")
			zombies.internal.zombies[oid]=nil
		end
	end
end

addhook("objectdamage","zombies.internal.objectdamage")
function zombies.internal.objectdamage(oid,dmg,id)
	if object(oid,"type")==30 then
		for _,id in pairs(player(0,"tableliving")) do
			for _,b in pairs(zombies.players[player(id,"usgn")].buffs) do
				if zombies.buffs[b[1]].zombiedmg then zombies.buffs[b[1]].zombiedmg(id,oid,dmg) end
			end
		end
		if zombies.internal.zombies[oid] then
			zombies.internal.zombies[oid][1]=zombies.internal.zombies[oid][1]-dmg
			if zombies.internal.zombies[oid][1]<=0 then
				local c=zombies.internal.zombies[oid]
				if zombies.drops[c[2]] then
					for _,d in pairs(zombies.drops[c[2]]) do
						local randchance = math.random(1,100)
						if randchance>100-d[1] then
							items.addToInventory(id,d[2],d[3])
							if d[3]==1 then
								msg2(id,"You found a "..items[d[2]][1].."!")
							else
								msg2(id,"You found "..d[3].." "..items[d[2]][1].."s!")
							end
						end
					end
				end
				zombies.internal.zombies[oid]=nil
				parse("damageobject "..oid.." "..object(oid,"health").." "..id)
			end
			return 1
		end
	end
end

addhook("hit","zombies.internal.hit")
function zombies.internal.hit(id,src,wpn,hdmg,admg,dmg)
	if zombies.players[player(id,"usgn")].safe then
		return 1
	end
	for _,b in pairs(zombies.players[player(id,"usgn")].buffs) do
		if zombies.buffs[b[1]].dmgrecv then zombies.buffs[b[1]].dmgrecv(id,src) end
	end
	if src~=0 then
		for _,b in pairs(zombies.players[player(src,"usgn")].buffs) do
			if zombies.buffs[b[1]].dmgdeal then zombies.buffs[b[1]].dmgdeal(src,id) end
		end
	end
	if not zombies.godmode[id] and not zombies.away[id] then
		zombies.damage(id,hdmg)
		zombies.addHunger(id,zombies.statsperhit[1])
		zombies.addThirst(id,zombies.statsperhit[2])
		zombies.addFatigue(id,zombies.statsperhit[3])
	end
	return 1
end

addhook("minute","zombies.internal.minute")
function zombies.internal.minute()
	for _,id in pairs(player(0,"tableliving")) do
		for _,b in pairs(zombies.players[player(id,"usgn")].buffs) do
			if zombies.buffs[b[1]].minute then zombies.buffs[b[1]].minute(id) end
		end
	end
end

addhook("movetile","zombies.internal.movetile")
function zombies.internal.movetile(id,x,y)
	parse("hudtxt ".. zombies.hudtxtoffset+2 .." \"X: "..x.."  Y: "..y.."\" 520 16 1")
	if not zombies.godmode[id] and not zombies.away[id] then
		zombies.addHunger(id,zombies.statspertile[1])
		zombies.addThirst(id,zombies.statspertile[2])
		zombies.addFatigue(id,zombies.statspertile[3])
	end
	for _,b in pairs(zombies.players[player(id,"usgn")].buffs) do
		if zombies.buffs[b[1]].movetile then zombies.buffs[b[1]].movetile(id,x,y) end
	end
	for _,sz in pairs(zombies.safezones) do
		if x>=sz[1] and y>=sz[2] and x<=sz[3] and y<=sz[4] then
			parse("hudtxt2 "..id.." ".. zombies.hudtxtoffset+7 .." \""..string.char(169).."000200000SAFE ZONE\" 320 200 1")
			if not zombies.players[player(id,"usgn")].safe then
				zombies.players[player(id,"usgn")].safe = true
			end
			break
		else
			parse("hudtxt2 "..id.." ".. zombies.hudtxtoffset+7 .." \"\" 320 200 1")
			if zombies.players[player(id,"usgn")].safe then
				zombies.players[player(id,"usgn")].safe = false
			end
		end
	end
end

addhook("walkover","zombies.internal.pickup")
function zombies.internal.pickup(id,iid,type)
	if not table.contains(zombies.players[player(id,"usgn")].weapons,type) then
		table.insert(zombies.players[player(id,"usgn")].weapons,type)
		parse("equip "..id.." "..type)
		parse("removeitem "..iid)
		return 1
	end
end

addhook("drop","zombies.internal.drop")
function zombies.internal.drop(id,iid,type)
	table.removevalue(zombies.players[player(id,"usgn")].weapons,type)
end

addhook("use","zombies.internal.use")
function zombies.internal.use(id)
	local x,y=player(id,"tilex"),player(id,"tiley")
	for _,a in pairs(zombies.actions) do
		if a[1]==x and a[2]==y then
			a[3](id,x,y)
		end
	end
	for _,b in pairs(zombies.beds) do
		if x*32+16>=b[1]-b[3] and y*32+16>=b[2]-b[4] and x*32+16<=b[1]+b[3] and y*32+16<=b[2]+b[4] then
			if not zombies.players[player(id,"usgn")].sleeping then
				zombies.players[player(id,"usgn")].sleeping = true
				if not zombies.players[player(id,"usgn")].sleepimg then
					zombies.players[player(id,"usgn")].sleepimg = image("gfx/sprites/block.bmp",320,240,2,id)
				end
				imagescale(zombies.players[player(id,"usgn")].sleepimg,20,15)
				imagecolor(zombies.players[player(id,"usgn")].sleepimg,0,0,0)
				imagealpha(zombies.players[player(id,"usgn")].sleepimg,0)
				tween_alpha(zombies.players[player(id,"usgn")].sleepimg,2000,1)
				zombies.players[player(id,"usgn")].speedmod = player(id,"speedmod")
				parse("speedmod "..id.." -100")
			else
				zombies.players[player(id,"usgn")].sleeping = false
				if zombies.players[player(id,"usgn")].sleepimg then
					tween_alpha(zombies.players[player(id,"usgn")].sleepimg,1000,0)
				end
				parse("speedmod "..id.." "..zombies.players[player(id,"usgn")].speedmod)
			end
		end
	end
end

addhook("say","zombies.internal.say")
function zombies.internal.say(id,t)
	local r=zombies.ranks[zombies.players[player(id,"usgn")].rank]
	if t=="!kills" then
		local nr=zombies.ranks[zombies.players[player(id,"usgn")].rank+1]
		msg2(id,"Enemy kills: "..zombies.players[player(id,"usgn")].kills.." ("..r[2]..")")
		msg2(id,"Next rank: "..nr[2].." with "..nr[1].." kills")
		return 1
	end
	if (t=="!rads") then
		local rads
		if (type(zombies.radspersec)=="table") then
			for _, rad in pairs(zombies.radspersec) do
				if time[1]>=rad[1] and time[1]<=rad[2] then
					rads = rad[3]
				end
			end
		elseif (type(zombies.radspersec)=="number") then
			rads = zombies.radspersec
		end
		if zombies.radsmults then
			for _, reg in pairs(zombies.radsmults) do
				if coordsInBounds(player(id,"tilex"),player(id,"tiley"),reg) then
					rads = rads * reg[5]
				end
			end
		end
		
		--msg2(id,"Rads per second: "..rads.." mSv")
		msg2(id,"Rads per second: "..zombies.players[player(id,"usgn")].radspersec.." mSv")
		msg2(id,"Your current radiation levels: "..zombies.players[player(id,"usgn")].rads.." mSv")
		return 1
	end
	if t=="rank" then
		return 0
	end
	if towns.internal.invited[id] then
		if t=="y" then
			table.insert(towns.internal.towns[towns.internal.invited[id][2]].members,player(id,"usgn"))
			msg2(id,"You have joined "..towns.internal.towns[towns.internal.invited[id][2]].."!")
			msg2(towns.internal.invited[id][1],player(id,"name").." has accepted your invitation.")
			towns.internal.invited[id] = nil
		end
		if t=="n" then
			msg2(towns.internal.invited[id][1],player(id,"name").." has declined your invitation.")
			towns.internal.invited[id] = nil
		end
	end
	if items.internal.trading[id] then
		if t=="y" then
			local did = items.internal.trading[id][1]
			local traded1 = items.giveItems(id,items.internal.trading[id][2])
			local traded2 = items.takeItems(id,items.internal.trading[id][3])
			local traded3, traded4
			if (traded1) then traded3 = items.takeItems(did,items.internal.trading[id][2]) end
			if (traded2) then traded4 = items.giveItems(did,items.internal.trading[id][3]) end
			if (traded1 and traded2 and traded3 and traded4) then
				msg2(id,"Trade completed successfully!")
				msg2(did,"Trade completed successfully!")
			else
				if not traded1 then
					msg2(id,"You don't seem to have enough inventory space!")
					msg2(did,player(id,"name").." doesn't seem to have enough inventory space!")
				end
				if not traded2 then
					msg2(id,"You don't seem to have enough items to give!")
					msg2(did,player(id,"name").." doesn't seem to have enough items to give!")
				end
				if not traded3 then
					msg2(id,player(did,"name").." doesn't seem to have enough items to give!")
					msg2(did,"You don't seem to have enough items to give!")
					if traded1 then
						items.takeItems(id,items.internal.trading[id][2])
					end
				end
				if not traded4 then
					msg2(id,player(did,"name").." doesn't seem to have enough inventory space!")
					msg2(did,"You don't seem to have enough inventory space!")
					if traded2 then
						items.giveItems(id,items.internal.trading[id][3])
					end
				end
			end
			
			items.internal.trading[id] = nil
		end
		if t=="n" then
			items.internal.trading[id] = nil
		end
	end
	local args=t:split()
	local cmd=args[1]
	table.remove(args,1)
	if cmd=="!towns" then
		if args[1]=="claim" then
			local plot=towns.internal.onplot[id]
			local name
			if args[2] then
				if args[2]:sub(1,1)=="\"" then
					for i=3,#args do
						if args[i]:sub(-1)=="\"" then
							name=table.concat(args," ",2,i)
							name=name:sub(2,-2)
						end
					end
				else
					name=args[2]
				end
				local taken=items.takeItems(id,zombies.townprice)
				if taken then
					zombies.foundTown(id,plot,name)
				end
			else
				msg2(id,string.char(169).."255000000You haven't specified a name for your settlement!")
			end
		end
		if args[1]=="build" then
			local buildings={}
			for _,b in pairs(zombies.buildingprice) do
				local desc=""
				buildings[#buildings+1]={buildingnames[_],items.itemList(b),function(id,item) msg(item[4]) zombies.players[player(id,"usgn")].building=item[4] end,_}
			end
			unimenu(id,true,{title="Build",items=buildings},1)
		end
		if args[1]=="change" then
			local town = towns.getTown(id)
			if args[2] == "owner" then
				if towns.getOwner(town) == player(id,"usgn") then towns.internal.towns[town].owner = tonumber(args[3]) end
			elseif args[2] == "name" then
				if towns.getOwner(town) == player(id,"usgn") then towns.internal.towns[town].name = args[3] end
			else
				if towns.getOwner(town) == player(id,"usgn") or (towns.isMember(id,town) and towns.getProperty(town,"membersedit")) then
					towns.setProperty(town,args[2],tonumber(args[3]) or toboolean(args[3]) or args[3])
				end
			end
		end
		if args[1]=="invite" then
			local town = towns.getTown(id)
			if towns.getOwner(town) == player(id,"usgn") or (towns.isMember(id,town) and towns.getProperty(town,"membersinvite")) then
				local did = tonumber(args[2])
				if did ~= id then
					towns.internal.invited[did] = {id,town}
					msg2(did,"You have been invited by "..player(id,"name").." to join "..towns.internal.towns[town].name.." (y/n)")
					msg2(id,"You invited "..player(id,"name").." to join "..towns.internal.towns[town].name)
				end
			end
		end
	end
	if cmd=="!cmd" then
		if table.contains(zombies.admins,player(id,"usgn")) then
			local cmd=args[1]
			table.remove(args,1)
			
			if cmd=="give" then
				local rid = tonumber(args[1])
				local iid = tonumber(args[2])
				local n = tonumber(args[3])
				items.addToInventory(rid,iid,n)
				return 1
			end
			
			if cmd=="god" then
				local did = tonumber(args[1])
				zombies.godmode[did] = not zombies.godmode[did]
				return 1
			end
			
			if cmd=="reset" then
				local did = tonumber(args[1])
				zombies.addHunger(did,100)
				zombies.addThirst(did,100)
				zombies.addFatigue(did,100)
				zombies.addHealth(did,100)
				return 1
			end
		end
	end
	if cmd=="!afk" then
		if not zombies.away[id] then
			zombies.away[id] = true
			msg("\169000255000"..player(id,"name").."\169255220000 is now away.")
			return 1
		end
	end
	msg(string.char(169).."000255000"..player(id,"name")..r[3]..string.char(169).."255220000: "..t)
	return 1
end

-- CONFIG --

dofile("sys/lua/zombies/config.lua")